﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;
using System.Diagnostics;

using MindFusion.RealTimeCharting.Wpf;

namespace RealTimeChart_Networking
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        DispatcherTimer dispatcherTimer;
        Stopwatch Tmr = new Stopwatch();
        Random rand;
        int clusterSize = 1;
        int currCount = 0;
        double y1 = 100, y2 = 100, y3 = 0.2;  // y# are filtered, randomly-varying values
        double PPS = 0;//points per second

        Series series1 = null;
        Series series2 = null;
        Series series3 = null;

        public Window1()
        {
            InitializeComponent();

            chart1.FrameLeftThickness = 50;
            chart1.FrameRightThickness = 100;
            chart1.FrameTopThickness = 100;
            chart1.FrameBottomThickness = 100;
            chart1.FastScrollMode = true;

            chart1.TooltipVisibility = Visibility.Visible;
            chart1.ShowFallbackTooltip = true;
                        
            chart1.LegendOrientation = System.Windows.Controls.Orientation.Horizontal;
            chart1.LegendWidth = double.NaN;
            chart1.LegendHeight = 30;      
            chart1.ShowLegend = true;

            chart1.MajorGridSizeY = 50;
            chart1.MajorGridSizeX = 25;
            chart1.MinorGridSizeX = 5;

            chart1.XAxis.Title = "Time";
            chart1.XAxis.PinLabels = false;                   

            Axis yAxis = new Axis();
            yAxis.Origin = -2000;
            yAxis.Length = 7000;
            yAxis.Interval = 500;
            yAxis.Title = "Server1";
            yAxis.TitleRotationAngle = -90.0;
            yAxis.TitleFontFamily = new FontFamily("Verdana");
            yAxis.LabelFontFamily = new FontFamily("Verdana");
            yAxis.TickLength = 5;
            yAxis.TitleOffset = 10;
            chart1.YAxisCollection.Add(yAxis);

            yAxis = new Axis();
            yAxis.Origin = -20;
            yAxis.Length = 140;
            yAxis.Interval = 10;
            yAxis.Title = "Server2";
            yAxis.TitleRotationAngle = -90.0;
            yAxis.TitleFontFamily = new FontFamily("Verdana");
            yAxis.LabelFontFamily = new FontFamily("Verdana");
            yAxis.TickLength = 5;
            yAxis.TitleOffset = 10;
            chart1.YAxisCollection.Add(yAxis);

            yAxis = new Axis();
            yAxis.Origin = -1;
            yAxis.Length = 2.8;
            yAxis.Interval = 0.2;
            yAxis.Title = "Server3";
            yAxis.TitleRotationAngle = -90.0;
            yAxis.TitleFontFamily = new FontFamily("Verdana");
            yAxis.LabelFontFamily = new FontFamily("Verdana");
            yAxis.TickLength = 5;
            yAxis.TitleOffset = 10;
            chart1.YAxisCollection.Add(yAxis);

            yAxis = new Axis();
            yAxis.Origin = 100;
            yAxis.Length = 1400;
            yAxis.Interval = 100;
            yAxis.Title = "Megabytes served";
            yAxis.TitleRotationAngle = -90.0;
            yAxis.TitleFontFamily = new FontFamily("Verdana");
            yAxis.LabelFontFamily = new FontFamily("Verdana");
            yAxis.TickLength = 5;
            yAxis.TitleOffset = 10;
            yAxis.LabelHorizontalAlignment = HorizontalAlignment.Right;
            chart1.Y2AxisCollection.Add(yAxis);

            chart1.XAxis.LabelFontFamily = new FontFamily("Verdana");
            chart1.XAxis.TitleFontFamily = new FontFamily("Verdana");

            series1 = new Series(chart1.YAxisCollection[0])
            {
                Stroke = new SolidColorBrush(Color.FromRgb(166, 46, 68)),
                Title = "Server 1",               
                ScatterType = ScatterType.None,
                TitleFontFamily = new FontFamily("Verdana"),
                TitleFontSize = 12
            
            };
            series2 = new Series(chart1.YAxisCollection[1])
            {
                Stroke = new SolidColorBrush(Color.FromRgb(9, 74, 10)),
                Fill = Brushes.Green,
                Title = "Server 2",               
                ScatterType = ScatterType.None,
                TitleFontFamily = new FontFamily("Verdana"),
                TitleFontSize = 12
            };
            //blue
            series3 = new Series(chart1.YAxisCollection[2])
            {
                Stroke = new SolidColorBrush(Color.FromRgb(40, 75, 89)),
                Fill = Brushes.Blue,
                Title = "Server 3",               
                ScatterType = ScatterType.None,
                TitleFontFamily = new FontFamily("Verdana"),
                TitleFontSize = 12
            };


            chart1.SeriesCollection.Add(series1);
            chart1.SeriesCollection.Add(series2);
            chart1.SeriesCollection.Add(series3);

            chart1.TooltipAxis = chart1.YAxisCollection[0];

            rand = new Random((int)DateTime.Now.Ticks);                
        }

        private void dispatcherTimer_Tick(object sender, EventArgs e)
        {
            chart1.FastScrollMode = true;
            Point[] points1 = new Point[clusterSize];
            Point[] points2 = new Point[clusterSize];
            Point[] points3 = new Point[clusterSize];

            double minNewX = currCount - 20;

            for (int i = 0; i < clusterSize; i++)
            {
                y1 = y1 * 0.93 + (rand.NextDouble() * 366) * 0.9;
                y2 = y2 * 0.4 + (rand.NextDouble() * 66) * 0.08;
                y3 = y3 * 0.6 + (rand.NextDouble() * 1.3) * 0.1;

                points1[i] = new Point(currCount + i - 20, y1 - 066);
                points2[i] = new Point(currCount + i - 10, y2 - 4 /*+ 066*/);
                points3[i] = new Point(currCount + i - 00, y3 - 0.1);
            }

            currCount += clusterSize;
            PPS = 1000d * currCount / Tmr.ElapsedMilliseconds;
            lblPoints.Content = (3 * currCount).ToString();
            //  TB_Rate.Text = "Points per second: " + PPS.ToString("#,0.0#");

            series1.Data.AddRange(points1);
            series2.Data.AddRange(points2);
            series3.Data.AddRange(points3);

            chart1.Commit(minNewX);

            DisposeOldData(series1);
            DisposeOldData(series2);
            DisposeOldData(series3);
        }

        private void DisposeOldData(Series series)
        {
            if (series.Data.Count > 100 &&
                series.Data[99].X < chart1.XAxis.Origin)
                series.Data.RemoveRange(0, 100);
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            //DispatcherTimer setup
            dispatcherTimer = new System.Windows.Threading.DispatcherTimer(DispatcherPriority.Send);
            dispatcherTimer.Tick += new EventHandler(dispatcherTimer_Tick);
            dispatcherTimer.Interval = TimeSpan.FromMilliseconds(40);
            dispatcherTimer.Start(); 

        }

        private void cbPinLabels_Checked(object sender, RoutedEventArgs e)
        {
            if (chart1 != null)
            {
                if (cbPinLabels.IsChecked.Value)              
                    chart1.XAxis.PinLabels = true;                  
                else               
                    chart1.XAxis.PinLabels = false;                 

             }
        }

        private void sldrSpeed_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            clusterSize = (int)sldrSpeed.Value;
        }

        private void RadioButton_Checked(object sender, RoutedEventArgs e)
        {
            if (chart1 == null)
                return;

            if (rb_server1.IsChecked.Value)
                chart1.TooltipAxis = chart1.YAxisCollection[0];
            else if (rb_server2.IsChecked.Value)
                chart1.TooltipAxis = chart1.YAxisCollection[1];
            else
                chart1.TooltipAxis = chart1.YAxisCollection[2];

        }

        private void CheckBox_Checked(object sender, RoutedEventArgs e)
        {
            if (sender == cbScatter1)          
                series1.ScatterType = ScatterType.Diamond;
            else if (sender == cbScatter2)
                series2.ScatterType = ScatterType.Square;
            else
                series3.ScatterType = ScatterType.Triangle;
        }

        private void CheckBox_Unchecked(object sender, RoutedEventArgs e)
        {
            if (sender == cbScatter1)
                series1.ScatterType = ScatterType.None;
            else if (sender == cbScatter2)
                series2.ScatterType = ScatterType.None;
            else
                series3.ScatterType = ScatterType.None;
        }
    }
}
